import { Injectable } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { fromEvent, Subject } from 'rxjs';
import { BrowserStorageService } from '../browser-storage/browser-storage.service';

export class WindowSize {
    isMobile: boolean;
    
    constructor(
        public width: number,
        public height: number,
    ) {
        this.isMobile = this.width <= 700;
    }
    
    static generate(): WindowSize {
        return new WindowSize(window.innerWidth, window.innerHeight);
    }
    
    update(): WindowSize {
        this.width = window.innerWidth;
        this.height = window.innerHeight;
        return this;
    }
}

@Injectable()
export class AppService {
    private sidebarHidden = false;
    private lang = 'en';
    private defaultLang = 'en';
    private dynamicToolbarComponents = [];
    langChange = new Subject<string>();
    size: WindowSize;
    sizeChange = new Subject<WindowSize>();
    sidebarHiddenChange = new Subject<boolean>();
    dynamicToolbarComponentsChange = new Subject<any[]>();

    constructor(
        private browserStorageService: BrowserStorageService,
        private translate: TranslateService,
    ) {
        this.configureResizeEvents();
        this.configureLanguageEvents();
        this.configureSidebarEvents();
    }

    configureSidebarEvents() {
        this.sidebarHiddenChange.subscribe(hidden => {
            this.sidebarHidden = hidden;
        });
    }

    configureResizeEvents() {
        this.sizeChange.subscribe(size => this.size = size);
        this.sizeChange.next(WindowSize.generate());
        fromEvent(window, 'resize').subscribe(e => {
            this.sizeChange.next(WindowSize.generate());
        });
    }

    configureLanguageEvents() {
        this.langChange.subscribe(lang => {
            this.lang = lang;
            this.browserStorageService.setItem('language', lang);
        });
        let language = this.browserStorageService.getItem('language');
        if (!language) {
            language = this.defaultLang;
        }
        this.translate.setDefaultLang(this.defaultLang);
        this.changeLang(language);
    }

    addDynamicToolbarComponent(component) {
        this.dynamicToolbarComponents.push(component);
        this.dynamicToolbarComponentsChange.next(this.dynamicToolbarComponents);
    }

    removeDynamicToolbarComponent(component) {
        this.dynamicToolbarComponents = this.dynamicToolbarComponents.filter(c => c !== component);
        this.dynamicToolbarComponentsChange.next(this.dynamicToolbarComponents);
    }

    setSidebarHidden(hidden: boolean) {
        this.sidebarHiddenChange.next(hidden);
    }

    getSidebarHidden() {
        return this.sidebarHidden;
    }
    
    changeLang(lang: string): void {
        if (lang !== this.lang) {
            this.translate.use(lang);
            this.langChange.next(lang);
        }
    }
    
    getLang(): string {
        return this.lang;
    }
    
    getLangs(): string[] {
        return [
            'pl',
            'en',
        ];
    }
}
